#include <wx/wx.h>
#include <wx/regex.h>
#include "ReservationEdition.h"


int idBtnOK = ::wxNewId();
int idBtnDefault = ::wxNewId();
int idBtnCancel = ::wxNewId();


BEGIN_EVENT_TABLE(ReservationEdition, wxDialog)
    EVT_BUTTON(idBtnOK, ReservationEdition::OnClickOK)
    EVT_BUTTON(idBtnDefault, ReservationEdition::OnClickDefault)

    EVT_CLOSE(ReservationEdition::OnCloseWindow)
END_EVENT_TABLE()


// the constructor is partly built with wxDialog's constructor
ReservationEdition::ReservationEdition(wxWindow* parent, wxWindowID id, const wxString& title, const wxPoint& pos, Reservation& reservation, const wxArrayString& macAddresses, const wxArrayString& ipAddresses)
: wxDialog(parent, id, title, pos, wxSize(200, 160))
, editedReservation(reservation) // initializes edited reservation
, macAddresses(macAddresses) // initializes others MAC addresses
, ipAddresses(ipAddresses) // initializes others IP addresses
{
    wxArrayString arraystrMACAddress; // authorized characters for MAC Address
    wxArrayString arraystrIPAddress; // authorized characters for IP Address
    wxTextValidator txtvldMACAddress(wxFILTER_INCLUDE_CHAR_LIST); // text validator for MAC Address
    wxTextValidator txtvldIPAddress(wxFILTER_INCLUDE_CHAR_LIST); // text validator for MAC Address

    //* definitions of MAC & IP Addresses text validators

    for ( int i = 0 ; i <= 9 ; i++ ) { // adds [0-9] characters to authorized characters for MAC Address
        wxString strTmp;
        strTmp.Printf("%d", i);
        arraystrMACAddress.Add(strTmp);
    }
    arraystrIPAddress = arraystrMACAddress; // copies MAC Address authorized characters list to IP Address's one (so, adds 0 to 9 characters to authorized characters for IP Address)

    arraystrMACAddress.Add("a"); // adds hexadecimal characters to authorized characters for MAC Address
    arraystrMACAddress.Add("b");
    arraystrMACAddress.Add("c");
    arraystrMACAddress.Add("d");
    arraystrMACAddress.Add("e");
    arraystrMACAddress.Add("f");
    arraystrMACAddress.Add("A");
    arraystrMACAddress.Add("B");
    arraystrMACAddress.Add("C");
    arraystrMACAddress.Add("D");
    arraystrMACAddress.Add("E");
    arraystrMACAddress.Add("F");
    arraystrMACAddress.Add(":"); // adds colon character to authorized characters for MAC Address

    arraystrIPAddress.Add("."); // adds period character to authorized characters for IP Address

    txtvldMACAddress.SetIncludes(arraystrMACAddress); // sets authorized characters for MAC Address
    txtvldIPAddress.SetIncludes(arraystrIPAddress); // sets authorized characters for IP Address


    //* drawing of GUI

    this->txtctrllblMACAddress = new wxTextCtrlLabelled(this, -1, "MAC Address", wxEmptyString, wxPoint(20,10), wxSize(110,20), 0, txtvldMACAddress); // displays MAC Address edit textcontrol
    this->txtctrllblMACAddress->wxTextCtrl::SetToolTip("Six [00-FF] hexadecimal numbers separated by colons\nFormat: xx:xx:xx:xx:xx:xx\nEx.: D8:00:11:CB:9F:C3");
    this->txtctrllblIPAddress = new wxTextCtrlLabelled(this, -1, "IP Address", wxEmptyString, wxPoint(20,50), wxSize(110,20), 0, txtvldIPAddress); // displays IP Address edit textcontrol
    this->txtctrllblIPAddress->wxTextCtrl::SetToolTip("Four [0-255] decimal numbers separated by periods\nFormat: A.B.C.D\nEx.: 192.168.1.1");

    this->btnOK = new wxButton(this, idBtnOK, "OK", wxPoint(10,100), wxSize(50,20)); // displays 'OK' button
    this->btnDefault = new wxButton(this, idBtnDefault, "Default", wxPoint(70,100), wxSize(50,20)); // displays 'Default' button
    this->btnCancel = new wxButton(this, idBtnCancel, "Cancel", wxPoint(130,100), wxSize(50,20)); // displays 'Cancel' button

    this->SetAffirmativeId(idBtnOK); // sets 'OK' button as affirmative button
    this->SetEscapeId(idBtnCancel); // sets 'Cancel' button as escape button

    this->InitializeReservation();
}


void ReservationEdition::InitializeReservation()
{
    this->txtctrllblMACAddress->SetValue(this->editedReservation.GetMACAddress()); // initializes MAC address value
    this->txtctrllblIPAddress->SetValue(this->editedReservation.GetIPAddress()); // initializes IP address value
}

void ReservationEdition::SetReservation(Reservation& reservation)
{
    reservation.SetMACAddress(this->txtctrllblMACAddress->GetValue()); // sets MAC address upon name field
    reservation.SetIPAddress(this->txtctrllblIPAddress->GetValue()); // sets IP address upon name field
}

bool ReservationEdition::CheckReservation(const Reservation& reservation)
{
    bool ok = true;
    wxArrayString errors;

    // check of MAC address
    // regular expression for MAC address (6 [00-FF] hexadecimal numbers separated by colons)
    // ([[:xdigit:]]{2}) : exactly 2 hexadecimal digits
    // (([[:xdigit:]]{2}):) : ...followed by ':'
    // (([[:xdigit:]]{2}):){5} : ...the whole exactly 5 times
    // ^(([[:xdigit:]]{2}):){5} : ...positioned at start
    // ([[:xdigit:]]{2}) : exactly 2 hexadecimal digits
    // ([[:xdigit:]]{2})$ : ...positioned at the end
    wxRegEx regxMACAddr("^(([[:xdigit:]]{2}):){5}([[:xdigit:]]{2})$");
    if ( !regxMACAddr.Matches(reservation.GetMACAddress()) ) {
        errors.Add("MAC address is invalid");
    }
    else {
        if ( this->macAddresses.Index(reservation.GetMACAddress()) != wxNOT_FOUND ) {
            errors.Add("MAC address '" + reservation.GetMACAddress() + "' is already assigned");
        }
    }

    // check of IP address
    // regular expression for IP address (4 [0-255] decimal numbers separated by periods)
    // [0-9]{1} : exactly 1 decimal digit ([0-9])
    // |[0-9]{2} : ...or exactly 2 decimal digits ([10-99])
    // |[0-1][0-9]{2} : ...or '0'/'1' followed by exactly 2 decimal digits ([000-199])
    // |2[0-4][0-9] : ...or 2 followed by '0'/'1'/'2'/'3'/'4' followed by 1 decimal digit ([200-249])
    // |25[0-5] : ...or '25' followed by '0'/'1'/'2'/'3'/'4'/'5' ([250-255])
    // (([0-9]{1}|[0-9]{2}|[0-1][0-9]{2}|2[0-4][0-9]|25[0-5])\\.) : ...the whole followed by '.' (quoted)
    // (([0-9]{1}|[0-9]{2}|[0-1][0-9]{2}|2[0-4][0-9]|25[0-5])\\.){3} : ...the whole exactly 3 times
    // ^(([0-9]{1}|[0-9]{2}|[0-1][0-9]{2}|2[0-4][0-9]|25[0-5])\\.){3} : ...positioned at start
    // ([0-9]{1}|[0-9]{2}|[0-1][0-9]{2}|2[0-4][0-9]|25[0-5]) : [0-9] + [10-99] + [000-199] + [200-249] + [250-255]
    // ([0-9]{1}|[0-9]{2}|[0-1][0-9]{2}|2[0-4][0-9]|25[0-5])$ : ...positioned at the end
    wxRegEx regxIPAddr("^(([0-9]{1}|[0-9]{2}|[0-1][0-9]{2}|2[0-4][0-9]|25[0-5])\\.){3}([0-9]{1}|[0-9]{2}|[0-1][0-9]{2}|2[0-4][0-9]|25[0-5])$");
    if ( !regxIPAddr.Matches(reservation.GetIPAddress()) ) {
        errors.Add("IP address is invalid");
    }
    else {
        if ( this->ipAddresses.Index(reservation.GetIPAddress()) != wxNOT_FOUND ) {
            errors.Add("IP address '" + reservation.GetIPAddress() + "' is already assigned");
        }
    }

    if ( errors.GetCount() != 0 ) {
        ok = false;
        wxString errorMsg = "Configuration settings are invalid:";
        for ( int i = 0 ; i < errors.GetCount() ; i++ ) {
            errorMsg += "\n- " + errors[i];
        }
        ::wxMessageBox(errorMsg, "Configuration error(s)", wxOK + wxICON_ERROR);
    }

    return ok;
}


void ReservationEdition::OnClickOK(wxCommandEvent& WXUNUSED(event))
{
    Reservation tempReservation;

    this->SetReservation(tempReservation);

    if ( this->CheckReservation(tempReservation) ) {
        this->editedReservation = tempReservation;
        this->EndModal(wxID_OK); // ends the dialog with 'ok' as return code
    }
}

void ReservationEdition::OnClickDefault(wxCommandEvent& WXUNUSED(event))
{
    this->InitializeReservation(); // sets back all fields to default
}

void ReservationEdition::OnCloseWindow(wxCloseEvent& WXUNUSED(event))
{
    this->EndModal(wxID_CANCEL); // ends the dialog with 'cancel' as return code
}
